#!ruby -Ks
# -*- mode: ruby; encoding: sjis -*-
# Last updated: <2014/12/13 18:01:34 +0900>

require 'dxruby'

# DXRuby BMFont `pNX
#
# ShoeBox ō쐬rbg}bvtHg`悷
# @see http://renderhjs.net/shoebox/
#
# @example
#     require 'dxruby'
#     require_relative 'bmfont'
#     bmfont = BMFont.new("bmfont.png", "bmfont.fnt")
#     Window.loop do
#       Window.draw_bmfont(100, 20, "Hello BMFont", bmfont)
#     end
#
# @version 0.0.2
#
class BMFont

  # tHg̊ei[\
  BMFontData = Struct.new(:image, :w, :h, :xofs, :yofs, :xadvance)

  # @return [Integer] ԃXy[X(Pʂ̓hbg)
  attr_accessor :spacing

  # @return [Integer] tHgc(Pʂ̓hbg)
  attr_reader :height

  # @return [Integer] tHgcl(Pʂ̓hbg)
  attr_reader :line_height

  # @return [Integer] tHgx[XCʒu
  attr_reader :baseofs

  # RXgN^
  # @param [String] imgfilename 摜t@C
  # @param [String] fntfilename zu(.fnt)t@C
  # @param [Integer] spacing    ԃXy[X
  def initialize(imgfilename, fntfilename, spacing = 0)
    img = Image.load(imgfilename)

    @bmfont_array = {}
    @spacing = spacing
    @height = 0
    @line_height = 0
    @baseofs = 0
    @chars_count = 0
    cnt = 0

    # .fntt@C1sǂݍŏ
    open(fntfilename) {|file|
      while l = file.gets
        l.chomp!
        if l =~ /^chars count=(\d+)$/
          # 擾
          @chars_count = $1.to_i
        elsif l =~ /^common (.+)$/
          # tHg̏cx[XCʒu擾
          p = $1
          p.split(' ').each do |s|
            if s=~ /^(\S+)=(\d+)$/
              case $1
              when "lineHeight"
                @line_height = $2.to_i
              when "base"
                @baseofs = $2.to_i
              end
            end
          end
        elsif l =~ /^char (.+) letter=\"(.)\"$/
          # 1̏擾
          str, c = $1, $2
          p = {}
          str.split(' ').each do |s|
            p[$1] = $2.to_i if s =~ /^(\S+)=(\d+)$/
          end

          id, x, y, w, h = p["id"], p["x"], p["y"], p["width"], p["height"]
          xs, ys, xa = p["xoffset"], p["yoffset"], p["xadvance"]
          cimg = img.slice(x, y, w, h) # 摜؂o
          @bmfont_array[id] = BMFontData.new(cimg, w, h, xs, ys, xa)
          @height = h if @height < h
          cnt += 1
        end
      end
    }
    img.dispose
  end

  # zƂĎw肵ۂɁABMFont̍\̂Ԃ
  # @param [Integer] index zY
  def [](index)
    return @bmfont_array[index]
  end
end

# WindowNXɒǉBMFont`p\bh
module Window

  # BMFont`
  # @param [Number] x `xW
  # @param [Number] y `yW
  # @param [String] str `敶
  # @param [Object] bmfont BMFontNX
  # @param [Number] z `D揇
  def self.draw_bmfont(x, y, str, bmfont, z = 0)
    str.each_byte do |code|
      x1 = x + bmfont[code].xofs
      y1 = y + bmfont[code].yofs - bmfont.line_height + bmfont.baseofs
      Window.draw(x1, y1, bmfont[code].image, z)
      # Window.drawBoxFill(x, y, x+2, y+2, C_RED)
      x += bmfont[code].xadvance + bmfont.spacing
    end
  end
end

# ImageNXɒǉBMFont`p\bh
class Image

  # BMFont`
  # @param [Number] x `xW
  # @param [Number] y `yW
  # @param [String] str `敶
  # @param [Object] bmfont BMFontNX
  def draw_bmfont(x, y, str, bmfont)
    str.each_byte do |code|
      x1 = x + bmfont[code].xofs
      y1 = y + bmfont[code].yofs - bmfont.line_height + bmfont.baseofs
      self.draw(x1, y1, bmfont[code].image)
      x += bmfont[code].xadvance + bmfont.spacing
    end
  end
end


if $0 == __FILE__

  #eXgR[h

  lst = [
    "!\"\#$%&'()*+,-./0",
    "123456789:;<=>?@",
    "A B C D EFGHIJKLMNOP",
    "QRSTUVWXYZ[\]^_`",
    "abcdefghijklmnop",
    "qrstuvwxyz{|}~"
  ]

  # BMFontNX𐶐
  bmfont = BMFont.new('mplus1p_heavy_0-export.png', 'mplus1p_heavy_0-export.fnt')

  # 摜ɕ`悵Ă݂
  image = Image.new(640, 240)
  x, y = 0, 0
  lst.each do |s|
    image.draw_bmfont(x, y, s, bmfont)
    y += bmfont.height
  end

  Window.bgcolor = [96, 96, 96]

  Window.loop do
    break if Input.keyPush?(K_ESCAPE)

    # ʂɕ`悵Ă݂
    x, y = 0, 0
    lst.each do |s|
      Window.draw_bmfont(x, y, s, bmfont)
      y += bmfont.height
    end

    Window.draw(0, 240, image)
  end
end
